require 'sketchup.rb'
Sketchup::require 'test_selection_observer/htmldialog.rb'
Sketchup::require 'test_selection_observer/product_selection_observer.rb'

module TestSelectionObserver

		@html_dialog = nil
		
		@id_to_product = {}

    @product_selection_observer = nil

    def self.select_instance(instance)
      selection = Sketchup.active_model.selection
      selection.clear
      selection.add(instance)
    end

    def self.onProductSelection(selection)
      selected_product_id = selection[0].attribute_dictionary("properties")["id"].to_s
      selected_product = @id_to_product[selected_product_id]
      @html_dialog.execute_script('updateSelectedProduct(%s)' % selected_product[:height])
    end

    def self.onProductSelectionCleared()
      @html_dialog.execute_script('noProductSelected()')
    end

		def self.create_cube_one(height)
			model = Sketchup.active_model
      model.start_operation('Create Cube', true)
      group = model.active_entities.add_group
      entities = group.entities
      points = [
        Geom::Point3d.new(0,   0,   0),
        Geom::Point3d.new(50.mm, 0,   0),
        Geom::Point3d.new(50.mm, 50.mm, 0),
        Geom::Point3d.new(0,   50.mm, 0)
      ]
      face = entities.add_face(points)
      face.pushpull(-height.mm)
			cint = group.to_component
      cint.set_attribute("properties", "id", 1);
			@id_to_product["1"] = {:instance => cint, :height=> height }
      select_instance(cint)
      add_product_selection_observer
      model.commit_operation
		end

		def self.create_cube_two(height)
			model = Sketchup.active_model
      model.start_operation('Create Cube', true)
      group = model.active_entities.add_group
      entities = group.entities
      points = [
        Geom::Point3d.new(60.mm,   0,   0),
        Geom::Point3d.new(110.mm, 0,   0),
        Geom::Point3d.new(110.mm, 110.mm, 0),
        Geom::Point3d.new(60.mm, 110.mm, 0)
      ]
      face = entities.add_face(points)
      face.pushpull(-height.mm)
			cint = group.to_component
      cint.set_attribute("properties","id", 2);
			@id_to_product["2"] = {:instance => cint, :height=> height }
      select_instance(cint)
      add_product_selection_observer
      model.commit_operation
		end

    def self.update_selected_product(height)
      selection = Sketchup.active_model.selection
      selected_product_id = selection[0].attribute_dictionary("properties")["id"].to_s
      selected_product = @id_to_product[selected_product_id]
      remove_product_selection_observer
      delete_product(selected_product[:instance])
      if(selected_product_id == 1)
        create_cube_one(height.to_f)
      else
        create_cube_two(height.to_f)
      end
      add_product_selection_observer
      
    end

    def self.delete_product(instance)
      instance.erase!
    end

    def self.remove_product_selection_observer
      Sketchup.active_model.selection.remove_observer(@product_selection_observer)
    end

    def self.add_product_selection_observer
      if(@product_selection_observer == nil)
        @product_selection_observer = ProductSelectionObserver.new
      end
      Sketchup.active_model.selection.add_observer(@product_selection_observer)
    end

		def self.register_callbacks(html_dialog)
			html_dialog.add_action_callback("createCubeOne") { |_action_context, height|
				create_cube_one(height)
			}
	
			html_dialog.add_action_callback("createCubeTwo") { |_action_context, height|
				create_cube_two(height)
		}

    html_dialog.add_action_callback("UpdateProduct") { |_action_context, height|
      update_selected_product(height)   
  }


		end

      menu = UI.menu('PlugIns').add_submenu("Test Selection Observer")
      menu.add_item('open html dialog') {
				@html_dialog = create_html_dialog
        @html_dialog.show
				register_callbacks(html_dialog)
      }
end  